/*
 * Copyright (c) 1991-2023 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and
 * its documentation is hereby granted, provided that this copyright
 * notice and license appear on all copies of the software.
 * 
 * STEP TOOLS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. STEP TOOLS
 * SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES.
 *
 * Author: David Loffredo (loffredo@steptools.com)
 */

#include <ifc_schema.h>
#include "make_units.h"

IfcUnitAssignment * make_units(
    RoseDesign * d
    )
{
    // Specify foot as the length unit, degree as the angle unit,
    // square feet as area. Since these are described in terms of the
    // SI units meter and radian, just trim this code if using meters
    //

    //----------------------------------------
    // LENGTH UNIT
    //----------------------------------------

    // SI unit for meter, we do not add this to the context, but we
    // use it to define the foot unit.  If you are using meters, just
    // add this to the context and omit the code below that build the
    // conversion-based unit instance.
    //
    IfcSIUnit * m = pnewIn(d) IfcSIUnit;
    m-> UnitType (IfcUnitEnum_LENGTHUNIT);
    m-> Prefix (IfcSIPrefix_NULL);
    m-> Name (IfcSIUnitName_METRE);

    // Foot is described as a conversion-based unit in based on the
    // meter SI unit.  Also show dimensional exponents for the basic
    // quantities (length, time, mass, etc) the unit describes.
    // 
    IfcMeasureWithUnit * mwu = pnewIn(d) IfcMeasureWithUnit;
    IfcMeasureValue * val = pnewIn(d) IfcMeasureValue;
    val-> _IfcLengthMeasure (0.3048);  // conversion factor ft->m

    mwu-> ValueComponent (pnewIn(d) IfcValue);
    mwu-> ValueComponent ()-> _IfcMeasureValue (val);

    mwu-> UnitComponent (pnewIn(d) IfcUnit);
    mwu-> UnitComponent ()-> _IfcNamedUnit (m);

    IfcConversionBasedUnit * ft = pnewIn(d) IfcConversionBasedUnit;
    ft-> Name ("FOOT");
    ft-> UnitType (IfcUnitEnum_LENGTHUNIT);
    ft-> Dimensions (pnewIn(d) IfcDimensionalExponents);
    ft-> Dimensions()-> LengthExponent(1);
    ft-> ConversionFactor (mwu);

    // Add foot to the unit context
    // The individual units are wrapped in a select type

    IfcUnit * u = pnewIn(d) IfcUnit;
    u-> _IfcNamedUnit (ft);

    IfcUnitAssignment * units = pnewIn(d) IfcUnitAssignment;
    units-> Units()-> add (u);


    //----------------------------------------
    // ANGLE UNIT
    //----------------------------------------

    // The SI angle unit is radians.  Do not add this to the context,
    // but use it to define the degree unit.
    //
    IfcSIUnit * rad = pnewIn(d) IfcSIUnit;
    rad-> UnitType (IfcUnitEnum_PLANEANGLEUNIT);
    rad-> Prefix (IfcSIPrefix_NULL);
    rad-> Name (IfcSIUnitName_RADIAN);

    // One Degree is Degree == PI/180 radians.  Create the measure
    // with unit to indicate this conversion factor.  We also need
    // dimensional exponents but these are all zero for angles.
    //
    mwu = pnewIn(d) IfcMeasureWithUnit;
    val = pnewIn(d) IfcMeasureValue;
    val-> _IfcPlaneAngleMeasure (0.01745329252);   // PI/180

    mwu-> ValueComponent (pnewIn(d) IfcValue);
    mwu-> ValueComponent ()-> _IfcMeasureValue (val);

    mwu-> UnitComponent (pnewIn(d) IfcUnit);
    mwu-> UnitComponent ()-> _IfcNamedUnit (rad);

    IfcConversionBasedUnit * deg = pnewIn(d) IfcConversionBasedUnit;
    deg-> Name ("DEGREE");
    deg-> UnitType (IfcUnitEnum_PLANEANGLEUNIT);
    deg-> Dimensions (pnewIn(d) IfcDimensionalExponents);
    deg-> ConversionFactor (mwu);

    // Add degree to the unit context
    u = pnewIn(d) IfcUnit;
    u-> _IfcNamedUnit (deg);
    units-> Units()-> add (u);


    //----------------------------------------
    // AREA UNIT
    //----------------------------------------

    // SI unit for square meters used to define square feet.  If you
    // are using square meters, just add this to the context and omit
    // the code below that build the conversion-based unit instance.
    //
    IfcSIUnit * sqm = pnewIn(d) IfcSIUnit;
    sqm-> UnitType (IfcUnitEnum_AREAUNIT);
    sqm-> Prefix (IfcSIPrefix_NULL);
    sqm-> Name (IfcSIUnitName_SQUARE_METRE);

    // Foot is described as a conversion-based unit in based on the
    // meter SI unit.  Also show dimensional exponents for the basic
    // quantities (length, time, mass, etc) the unit describes.
    // 
    mwu = pnewIn(d) IfcMeasureWithUnit;
    val = pnewIn(d) IfcMeasureValue;
    val-> _IfcAreaMeasure (0.09290304);  // convert sqft->sqm

    mwu-> ValueComponent (pnewIn(d) IfcValue);
    mwu-> ValueComponent ()-> _IfcMeasureValue (val);

    mwu-> UnitComponent (pnewIn(d) IfcUnit);
    mwu-> UnitComponent ()-> _IfcNamedUnit (sqm);

    IfcConversionBasedUnit * sqft = pnewIn(d) IfcConversionBasedUnit;
    sqft-> Name ("SQUARE FOOT");
    sqft-> UnitType (IfcUnitEnum_AREAUNIT);
    sqft-> Dimensions (pnewIn(d) IfcDimensionalExponents);
    sqft-> Dimensions()-> LengthExponent(2);
    sqft-> ConversionFactor (mwu);

    // Add unit context
    u = pnewIn(d) IfcUnit;
    u-> _IfcNamedUnit (ft);
    units-> Units()-> add (u);


    return units;
}
